/************************************************************************************
					 _     _           _     _       _     _   
					| |   (_)         | |   (_)     | |   | |  
					| |    ___   _____| |    _  __ _| |__ | |_ 
					| |   | \ \ / / _ \ |   | |/ _` | '_ \| __|
					| |___| |\ V /  __/ |___| | (_| | | | | |_ 
					\_____/_| \_/ \___\_____/_|\__, |_| |_|\__|
												__/ |          
BETA Firmware 								   |___/
							
DSP effects for LiveLight V2 Firmware 						 		 ________   __
Written by Henri Rantanen											|  ____\ \ / /
																	| |__   \ V / 
Date		16.10.2011												|  __|   > <  
Version		1.0														| |     / . \ 
Target		PCB Version 5, Atmel ATMEGA32-16U						|_|    /_/ \_\

************************************************************************************/
#include "settings.h"

void FadeIn(uint8_t Brightness)					// Fade all lights on
{
	cli();										// Disable all interrupts
	
	volatile uint8_t i = 0;						// Brightness index variable
	volatile uint8_t j = 0;						// Channel number index variable
	
	for (i = 0; i < Channels; i++)				// Set brightness to 0
	{
		Color_Array[i] = 0;
	}
	
	for (i = 0; i < 255; i++)					// Run loop 255 times
	{
		for (j = 1; j < (Channels + 1); j++)	// Decrease brightness of all channels
		{	
			if (Color_Array[j] < Brightness)	// If brightness is not 0 already
			{
				Color_Array[j]++;				// Decrease brightness
			}
		}
		
		for (j = 0; j < 255; j++)				// Run a full PWM cycle
		{
			switch_routine(Color_Array);		// Run one PWM Switch cycle
		}
	}
	
	sei();										// Enable all interrupts
	return;
}

void FadeOut(void)								// Fade all lights off
{
	volatile uint8_t i = 0;						// Brightness index variable
	volatile uint8_t j = 0;						// Channel number index variable
	
	cli();										// Disable all interrupts
	
	for (i = 0; i < 255; i++)					// Run loop 255 times
	{
		for (j = 0; j < (Channels + 1); j++)	// Decrease brightness of all channels
		{	
			if (Color_Array[j] > 0)				// If brightness is not 0 already
			{
				Color_Array[j]--;				// Decrease brightness
			}
		}
		
		for (j = 0; j < 255; j++)				// Run a full PWM cycle
		{
			switch_routine(Color_Array);		// Run one PWM Switch cycle
		}
	}
	
	sei();										// Enable all interrupts
}

void ResetShutdownTimer(void)					// Reset time to Off-line
{
	if (TIMSK & ~0x01)
	{
		TIMSK |= (1 << TOIE0);		// Re-enable shut-down timer after "Off-line"
	}
	
	TCNT0 = 0;						// Reset Shut-Down timer
}

volatile uint8_t NoInputCounter = 0;			// Off-line timer calculator

ISR(TIMER0_OVF_vect)							// TIMER0 Overflow interrupt
{		
	if (NoInputCounter > ShutDownDelay)			// If counter has reached max. value
	{
		TIMSK = TIMSK & ~(1 << TOIE0);			// Disable TIMER0 interrupt
		NoInputCounter = 0;						// Reset shutdown counter	
		FadeOut();								// Kill all lights
		FadeIn(64);								// All lights to gray
	}
	else
	{
		NoInputCounter++;						// Grow Off-line counter
	}
}

