/************************************************************************************
					 _     _           _     _       _     _   
					| |   (_)         | |   (_)     | |   | |  
					| |    ___   _____| |    _  __ _| |__ | |_ 
					| |   | \ \ / / _ \ |   | |/ _` | '_ \| __|
					| |___| |\ V /  __/ |___| | (_| | | | | |_ 
					\_____/_| \_/ \___\_____/_|\__, |_| |_|\__|
												__/ |          
BETA Firmware 								   |___/
							
LiveLight USB Interface with ATMEGA32
Written by Henri Rantanen 

Date		6.9.2011
Version		2.0
Target		PCB Version 5, Atmel ATMEGA32-16U

************************************************************************************/
#include "settings.h"

void Initialize(void)
{
	// Initialize TIMER0
	TCCR0 |= (1 << CS02)|(0 << CS01)|(1 << CS00);	// Timer pre-scaler 1/1024
	TIMSK |= (0 << TOIE0);							// Disable Overflow Interrupt
	TCNT0 = 0;										// Initialize Counter
	
	// Initialize TIMER2
	TCCR2 |= (0 << CS22)|(1 << CS21)|(0 << CS20);	// Timer pre-scaler 1/8
	TIMSK |= (1 << TOIE2);							// Enable Overflow Interrupt
	TCNT2 = 0;										// Initialize Counter
	
	//Initialize UART
	UBRRH = 0;										// UART Speed HIGH
	UBRRL = ((F_CPU / 16) / BAUD_RATE) - 1; 		// UART Speed LOW
	UCSRB |= (1 << RXEN);                 			// Enable RX
	UCSRB |= (1 << TXEN);							// Enable TX
	UCSRC = 0x86;									// Character size 8 bits
	UCSRB |= 0x80;									// Enable RX interrupt

	// Initialize output pins
	DDRA = 0xFF;									// PORT A 1111 1111			
	DDRB = 0xFF;									// PORT B 1111 1111
	DDRC = 0xFF;									// PORT C 1111 1111
	DDRD = 0xFC;									// PORT D 1111 1100
	
	sei();											// Enable global interrupts
}

int main(void)
{	 
	// Initialize the MCU
	Initialize();
	
	// Starting color 
	FadeIn(64);
	
	while(1)
    {
		// Main loop is empty
 		// Interrupts do all the work!
    }
}