/************************************************************************************
					 _     _           _     _       _     _   
					| |   (_)         | |   (_)     | |   | |  
					| |    ___   _____| |    _  __ _| |__ | |_ 
					| |   | \ \ / / _ \ |   | |/ _` | '_ \| __|
					| |___| |\ V /  __/ |___| | (_| | | | | |_ 
					\_____/_| \_/ \___\_____/_|\__, |_| |_|\__|
												__/ |          
BETA Firmware 								   |___/
							
PWM Algorithm for LiveLight V2 Firmware					 _______          ____  __ 
Written by Henri Rantanen								|  __ \ \        / /  \/  |
														| |__) \ \  /\  / /| \  / |
Date		15.10.2011									|  ___/ \ \/  \/ / | |\/| |
Version		2.0											| |      \  /\  /  | |  | |
Target		PCB Version 5, Atmel ATMEGA32-16U			|_|       \/  \/   |_|  |_|

************************************************************************************/
#include "settings.h"

//Global variables
volatile uint8_t PWM = 0;

// PWM Function
void switch_routine(volatile uint8_t brightness[])
{		
		// Right Bottom RGB Channel
		if(brightness[1] > PWM)  		// 1. Channel switch
		{
			PORTA |= 0x04;
		}
		else
		{
			PORTA = PORTA & ~0x04;
		}

		if(brightness[2] > PWM) 		// 2. Channel switch
		{
			PORTA |= 0x10;
		}
		else
		{
			PORTA = PORTA & ~0x10;
		}

		if(brightness[3] > PWM) 		// 3. Channel switch
		{
			PORTA |= 0x8;
		}
		else
		{
			PORTA = PORTA & ~0x8;
		}


		// Right Top RGB Channel
		if(brightness[4] > PWM) 		// 4. Channel switch
		{
			PORTB |= 0x04;
		}
		else
		{
			PORTB = PORTB & ~0x04;
		}
		if(brightness[5] > PWM) 		// 5. Channel switch
		{
			PORTA |= 0x02; 
		}
		else
		{
			PORTA = PORTA & ~0x02;
		}
		
		if(brightness[6] > PWM) 		// 6. Channel switch
		{
			PORTB |= 0x02; 
		}
		else
		{
			PORTB = PORTB & ~0x02;
		}
	
		
		// Top Right RGB Channel
		if(brightness[7] > PWM) 		// 7. Channel switch
		{
			PORTA |= 0x01;
		}
		else
		{
			PORTA = PORTA & ~0x01;
		}
		
		if(brightness[8] > PWM) 		// 8. Channel switch
		{
			PORTB |= 0x08;
		}
		else
		{
			PORTB = PORTB & ~0x08;
		}
		
		if(brightness[9] > PWM) 		// 9. Channel switch
		{
			PORTB |= 0x01;
		}		
		else
		{
			PORTB = PORTB & ~0x01;
		}


		// Top Center RGB Channel
		if(brightness[10] > PWM) 		// 10. Channel switch
		{
			PORTD |= 0x08;
		}
		else
		{
			PORTD = PORTD & ~0x08;
		}
		
		if(brightness[11] > PWM) 		// 11. Channel switch
		{
			PORTB |= 0x10;				// For PCB V5
			//PORTD |= 0x02;			// For PCB V4
		}
		else
		{
			PORTB = PORTB & ~0x10;		// For PCB V5
			//PORTD = PORTD & ~0x02;	// For PCB V4
		}
		
		if(brightness[12] > PWM)		// 12. Channel switch
		{
			PORTD |= 0x04;
		}
		else
		{
			PORTD = PORTD & ~0x04;
		}


		// Top Left RGB Channel
		if(brightness[13] > PWM) 		// 13. Channel switch
		{
			PORTD |= 0x10;
		}
		else
		{
			PORTD = PORTD & ~0x10;
		}
		
		if(brightness[14] > PWM) 		// 14. Channel switch
		{
			PORTD |= 0x40;
		}
		else
		{
			PORTD = PORTD & ~0x40;
		}

		if(brightness[15] > PWM)		// 15. Channel switch
		{
			PORTD |= 0x20;
		}
		else
		{
			PORTD = PORTD & ~0x20;
		}


		// Left Top RGB Channel
		if(brightness[16] > PWM) 		// 16. Channel switch
		{
			PORTC |= 0x01;
		}
		else
		{
			PORTC = PORTC & ~0x01;
		}
		
		if(brightness[17] > PWM) 		// 17. Channel switch
		{
			PORTD |= 0x80;
		}
		else
		{
			PORTD = PORTD & ~0x80;
		}

		if(brightness[18] > PWM)		// 18. Channel switch
		{
			PORTC |= 0x02;
		}
		else
		{
			PORTC = PORTC & ~0x02;
		}


		// Left Bottom RGB Channel

		if(brightness[19] > PWM) 		// 19. Channel switch
		{
			PORTC |= 0x10;
		}
		else
		{
			PORTC = PORTC & ~0x10;
		}
		
		if(brightness[20] > PWM) 		// 20. Channel switch
		{
			PORTC |= 0x04;
		}
		else
		{
			PORTC = PORTC & ~0x04;
		}
		
		if(brightness[21] > PWM)		// 21. Channel switch
		{
			PORTC |= 0x08;
		}
		else
		{
			PORTC = PORTC & ~0x08;
		}
		

		// Bottom Left RGB Channel
		if(brightness[22] > PWM) 		// 22. Channel switch
		{
			PORTC |= 0x20;
		}
		else
		{
			PORTC = PORTC & ~0x20;
		}
		
		if(brightness[23] > PWM) 		// 23. Channel switch
		{
			PORTC |= 0x80;
		}
		else
		{
			PORTC = PORTC & ~0x80;
		}
		
		if(brightness[24] > PWM)		// 24. Channel switch
		{
			PORTC |= 0x40;
		}
		else
		{
			PORTC = PORTC & ~0x40;
		}	
		

		//Bottom Right RGB Channel
		if(brightness[25] > PWM)		// 25. Channel switch
		{
			PORTA |= 0x20;
		}
		else
		{
			PORTA = PORTA & ~0x20;
		}

		if(brightness[26] > PWM)		// 26. Channel switch
		{
			PORTA |= 0x40;
		}
		else
		{
			PORTA = PORTA & ~0x40;
		}

		if(brightness[27] > PWM)		// 27. Channel switch
		{
			PORTA |= 0x80;
		}
		else
		{
			PORTA = PORTA & ~0x80;
		}

		// Grow PWM index with 1 or 2 bits
		// Mode of operation is defined in the settings
		if (PWM_Mode)
		{
			// 8-bit mode
			PWM++;
		} 
		else
		{
			// 7-bit mode
			PWM++;
			PWM++;
		}
}

// Testing pattern
void test_pattern(void)
{
	Color_Array[1]	= 32;	Color_Array[2] = 128;	Color_Array[3] = 128;	// Right Bottom
	Color_Array[4]	= 32;	Color_Array[5] = 255;	Color_Array[6] = 32;	// Right Top
	Color_Array[7]	= 128;	Color_Array[8] = 128;	Color_Array[9] = 32;	// Top Right
	Color_Array[10] = 255;	Color_Array[11] = 32;	Color_Array[12] = 32;	// Top Center
	Color_Array[13] = 128;	Color_Array[14] = 128;	Color_Array[15] = 32;	// Top Left
	Color_Array[16] = 32;	Color_Array[17] = 255;	Color_Array[18] = 32;	// Left Top
	Color_Array[19] = 32;	Color_Array[20] = 128;	Color_Array[21] = 128;	// Left Bottom
	Color_Array[22] = 32;	Color_Array[23] = 32;	Color_Array[24] = 255;	// Bottom Left
	Color_Array[25] = 32;	Color_Array[26] = 32;	Color_Array[27] = 255;	// Bottom Right
	
	return;
}

// TIMER2 Overflow interrupt
ISR(TIMER2_OVF_vect)
{
	switch_routine(Color_Array);	// Run PWM Switching algorithm once
	TCNT2 = 250;					// Re-calibrate PWM timer
}