/************************************************************************************
					 _     _           _     _       _     _   
					| |   (_)         | |   (_)     | |   | |  
					| |    ___   _____| |    _  __ _| |__ | |_ 
					| |   | \ \ / / _ \ |   | |/ _` | '_ \| __|
					| |___| |\ V /  __/ |___| | (_| | | | | |_ 
					\_____/_| \_/ \___\_____/_|\__, |_| |_|\__|
												__/ |          
BETA Firmware 								   |___/
							
USART RX Input code for LiveLight V2 Firmware 						 _____  __   __
Written by Henri Rantanen											|  __ \ \ \ / /
																	| |__) | \ V /
Date		16.10.2011												|  _  /   > <  
Version		2.0														| | \ \  / . \ 
Target		PCB Version 5, Atmel ATMEGA32-16U						|_|  \_\/_/ \_\

************************************************************************************/
#include "settings.h"

/* Data input protocol example
	+--------+-------+---------+--------+-------+---------+--------+-------+----------/
	| Header | Red 1 | Green 1 | Blue 1 | Red 2 | Green 2 | Blue 2 | Red 3 | Green3  /
	+--------+-------+---------+--------+-------+---------+--------+-------+--------/
*/

ISR(USART_RXC_vect)								// UART Receive interrupt
{	
	ResetShutdownTimer();						// Reset shutdown timer
	
	volatile uint8_t i;							// Index variable
	volatile uint8_t rxBuffer[Channels + 1];	// Receiver buffer for input
	
 	if(i > Channels)							// Reset index if overflowing out of range						
	{
		i = 0;									// Reset index
	}
	
	rxBuffer[i] = UDR;							// Read USART to buffer

	if (rxBuffer[0] == PREFIX)					// Check if Header-byte is OK
	{ 
		if(rxBuffer[i] <= Blacklevel)			// Apply minimum brightness to input
		{
			Color_Array[i] = Blacklevel;		// Set brightness to defined
		}
		else
		{
			Color_Array[i] = rxBuffer[i];		// Save brightness to new value
		}
		i++;									// Grow index
		
	}
	else
	{
		i = 0;									// Reset index
	}		
}

