/************************************************************************************
					 _     _           _     _       _     _   
					| |   (_)         | |   (_)     | |   | |  
					| |    ___   _____| |    _  __ _| |__ | |_ 
					| |   | \ \ / / _ \ |   | |/ _` | '_ \| __|
					| |___| |\ V /  __/ |___| | (_| | | | | |_ 
					\_____/_| \_/ \___\_____/_|\__, |_| |_|\__|
												__/ |          
BETA Firmware 								   |___/
							
Settings file for LiveLight V2 Firmware
Written by Henri Rantanen 

Date		11.10.2011
Version		2.0
Target		PCB Version 5, Atmel ATMEGA32-16U

************************************************************************************/

#ifndef SETTINGS_H_
#define SETTINGS_H_

// Module parameters
#define Channels			27						// Amount of output channels 
#define F_CPU 				14745600				// Clock speed 14,7456 MHz
#define BAUD_RATE 			9600    				// Set Baud-rate
#define PREFIX				0x5A					// Header byte
#define ShutDownDelay		0x0A					// Delay in witch to turn off LED's
#define Blacklevel			0x0						// Minimum brightness of the output 0-255

#define PWM_Mode			0x00					// 0 = 7-bit PWM,	1 = 8-bit PWM.
//	8-bit mode				16777216 colors			// Full output				
//	7-bit mode				 2097152 colors			// Double speed
												
// Header files
#include <avr/io.h>
#include <avr/interrupt.h>

// Global variables
volatile uint8_t Color_Array[1 + Channels];			// Header and main color table

// Function prototypes
void ResetShutdownTimer(void);						// Resets countdown to off-line mode
void FadeOut(void);									// Fades all channels off
void TX_Char(volatile uint8_t data);				// Transmits character to PC
void test_pattern(void);							// Saves testing pattern to Color_Array
void Initialize(void);								// Initializes the C
void switch_routine(volatile uint8_t brightness[]);	// PWM Switch function

#endif /* SETTINGS_H_ */